from .metaData import MetaData

class Meeting(object):
    def __init__(self, xml):
        self.__meetingId = xml["meetingID"]
        self.__meetingName = xml["meetingName"]
        self.__creationTime = float(xml["createTime"])
        self.__creationDate = xml["createDate"]
        self.__voiceBridge = xml["voiceBridge"]
        self.__dialNumber = xml["dialNumber"]
        self.__attendeePassword = xml["attendeePW"]
        self.__moderatorPassword = xml["moderatorPW"]
        self.__hasBeenForciblyEnded = xml["hasBeenForciblyEnded"] == 'true'
        self.__isRunning = xml["running"] == 'true'
        self.__participantCount = int(xml["participantCount"])
        self.__listenerCount = int(xml["listenerCount"])
        self.__voiceParticipantCount = int(xml["voiceParticipantCount"])
        self.__videoCount = int(xml["videoCount"])
        self.__duration = int(xml["duration"])
        self.__hasUserJoined = xml["hasUserJoined"] == 'true'
        self.__meta = MetaData(xml["metadata"])

    def get_meetingid(self):
        return self.__meetingId

    def get_meetingname(self):
        return self.__meetingName

    def get_createtime(self):
        return self.__creationTime

    def get_createdate(self):
        return self.__creationDate

    def get_voicebridge(self):
        return self.__voiceBridge

    def get_dialnumber(self):
        return self.__dialNumber

    def get_attendeepw(self):
        return self.__attendeePassword

    def get_moderatorpw(self):
        return self.__moderatorPassword

    def get_participantcount(self):
        return self.__participantCount

    def get_listenercount(self):
        return self.__listenerCount

    def get_voiceparticipantcount(self):
        return self.__voiceParticipantCount

    def get_videocount(self):
        return self.__videoCount

    def get_duration(self):
        return self.__duration

    def has_user_joined(self):
        return self.__hasUserJoined

    def has_been_forcibly_ended(self):
        return self.__hasBeenForciblyEnded




import sys

if sys.version_info[0] == 2:
    from abc import ABCMeta

    class BaseResponse(dict):
        __metaclass__ = ABCMeta

        def __init__(self, rawXml):
            dict.__init__(self, xml=rawXml)
            self.rawXml = rawXml

        def get_field(self, label):
            return self.rawXml[label]

        def get_return_code(self):
            return self.get_field("returncode")

        def get_message_key(self):
            return self.get_field("messageKey")

        def get_message(self):
            return self.get_field("message")
else:
    from abc import ABC

    class BaseResponse(ABC, dict):
        def __init__(self, rawXml):
            dict.__init__(self, xml=rawXml)
            self.rawXml = rawXml

        def get_field(self, label):
            return self.rawXml[label]

        def get_return_code(self):
            return self.get_field("returncode")

        def get_message_key(self):
            return self.get_field("messageKey")

        def get_message(self):
            return self.get_field("message")



class BBBException(Exception):
    HTTP_ERROR = "httpError"
    NOT_FOUND = "notFound"
    NO_ACTION = "noActionSpecified"
    ID_NOT_UNIQUE = "idNotUnique"
    NOT_STARTED = "notStarted"
    ALREADY_ENDED = "alreadyEnded"
    INTERNAL_ERROR = "internalError"
    UNREACHABLE_SERVER = "unreachableServer"
    INVALID_RESPONSE = "invalidResponse"
    GENERAL_ERROR = "generalError"
    CHECKSUM_ERROR = "checksumError"
    INVALID_RECORDING_STATE = "invalidRecordingState"
    CONFIG_XML_ERROR = "configXMLError"
    XML_SYNTAX_ERROR = "XMLSyntaxError"


    def __init__(self, key, message):
        Exception.__init__(self, message)
        self.key = key



from .base import BaseResponse
from datetime import datetime

class CreateMeetingResponse(BaseResponse):
    def get_meetingid(self):
        return self.get_field("meetingID")

    def get_attendee_pw(self):
        return self.get_field("attendeePW")

    def get_moderator_pw(self):
        return self.get_field("moderatorPW")

    def get_create_time(self):
        return datetime.fromtimestamp(int(self.get_field("createTime")) / 1e3).strftime('%Y-%m-%d %H:%M:%S')

    def has_been_forcibly_ended(self):
        field = self.get_field("hasBeenForciblyEnded")
        return field == "true" or (isinstance(field, bool) and field)

    def get_voice_bridge(self):
        return self.get_field("voiceBridge")

    def get_dial_number(self):
        return self.get_field("dialNumber")


from .base import BaseResponse

class IsMeetingRunningResponse(BaseResponse):
    def is_meeting_running(self):
        field = self.get_field("running")
        return field == "true" or (isinstance(field, bool) and field)


from .base import BaseResponse
from ..core.record import Record


class GetRecordingsResponse(BaseResponse):

    def get_recordings(self):
        recordings = []

        try:
            if self.get_message_key() == "noRecordings":
                return []
        except KeyError:
            pass

        for recordXml in self.get_field("recordings")["recording"]:
            recordings.append(Record(recordXml))

        return recordings
